//
//  ContentView.swift
//  ListImageIssue
//
//  Created by Thomas Goddard on 7/15/21.
//

import SwiftUI

struct ContentView: View {
    @EnvironmentObject var modelData: ModelData
    
    var body: some View {
        NavigationView {
            List {
                ForEach(modelData.items, id: \.self) { item in
                    LargeItem(item: item)
                }
                .listRowInsets(EdgeInsets())
                .listRowSeparator(.hidden)
            }
            .navigationTitle("Items")
        }
    }
}

extension String {
    func CGFloatValue() -> CGFloat? {
        guard let doubleValue = Double(self) else {
            return nil
        }
        
        return CGFloat(doubleValue)
    }
}

struct LargeItem: View {
    var item: [String: String]
    var showDivider = false
    let imageWidth = UIScreen.main.bounds.width
    
    var body: some View {
        var attrStr = AttributedString(item["itemID"]!)
        attrStr.link = URL(string: item["url"]!)
        return VStack {
            ZStack(alignment: .center) {
                if let imageUrl = item["url"],
                   let mediaSizeWidthString = item["width"],
                   let mediaSizeWidth = mediaSizeWidthString.CGFloatValue(),
                   let mediaRatio = imageWidth / mediaSizeWidth,
                   let mediaSizeHeightString = item["height"],
                   let mediaSizeHeight = mediaSizeHeightString.CGFloatValue(),
                   let mediaHeight = mediaRatio*mediaSizeHeight,
                   !mediaHeight.isNaN {
                    AsyncImage(url: URL(string: imageUrl)) { phase in
                        switch phase {
                        case .empty:
                            Color.blue
                        case .success(let image):
                            ZStack(alignment: .center) {
                                image
                                    .renderingMode(.original)
                                    .resizable()
                                    .aspectRatio(contentMode: .fit)
                                Color
                                    .black
                                    .opacity(0.5)
                                    .frame(height: 30, alignment: .center)
                                    .background(.thinMaterial)
                                Text(attrStr)
                            }
                        case .failure(let error):
                            ZStack(alignment: .center) {
                                Color.red
                                ZStack {
                                    Color
                                        .black
                                        .opacity(0.5)
                                        .frame(height: 60, alignment: .center)
                                        .background(.thinMaterial)
                                    VStack {
                                        Text(attrStr)
                                        Text("Error: \(error.localizedDescription)")
                                    }
                                }
                            }
                        @unknown default:
                            fatalError()
                        }
                    }
                    .mask(RoundedRectangle(cornerRadius: 10))
                    .frame(minHeight: mediaHeight, alignment: .center)
                    .padding(.top, 8)
                    .task {
                        debugPrint("Top level task called: \(imageUrl)")
                    }
                }
            }
        }
        .animation(.easeInOut, value: true)
    }
}

struct ContentView_Previews: PreviewProvider {
    static var previews: some View {
        ContentView()
    }
}
